SystemOrganization addCategory: #'EjerciciosPoo-TusLibros'!

Error subclass: #BookNotInCatalogException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

Error subclass: #NotAValidQuantityException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

Object subclass: #Cart
	instanceVariableNames: 'id contents marketplace'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/12/2012 20:01'!
withId: aNumber belongingTo: aMarketplace
	 ^(((Cart new) setId: aNumber) setContents: Bag new) setMarketplace: aMarketplace.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/12/2012 20:06'!
addBook: anISBN withQuantity: aNumber
	
	aNumber > 0 ifFalse: [NotAValidQuantityException new signal].
	(marketplace isAListedBook: anISBN) ifFalse: [BookNotInCatalogException new signal.].
	contents add: anISBN withOccurrences: aNumber.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/12/2012 19:11'!
listContents
	^ contents.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/11/2012 21:49'!
setContents: aContentsCollection
	
	contents := aContentsCollection.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/11/2012 21:58'!
setId: aNumber 
	
	id := aNumber ! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/12/2012 20:02'!
setMarketplace: aMarketplace 
	
	marketplace := aMarketplace  ! !

Object subclass: #Marketplace
	instanceVariableNames: 'carts catalogue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Marketplace class methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/11/2012 21:59'!
withBooks: aCollectionOfBooks

	^((Marketplace new) setBooks: aCollectionOfBooks) setCarts: OrderedCollection new.! !

!Marketplace methodsFor: 'managing carts' stamp: 'RavasiAronson 6/12/2012 20:09'!
createCart
	|id cart|
	
	id := carts size.
	
	cart:= Cart withId: id + 1 belongingTo: self.
	
	carts add: cart.
	
	^cart.! !

!Marketplace methodsFor: 'managing carts' stamp: 'RavasiAronson 6/12/2012 20:34'!
getCart:aCartID

	 ^ carts at: aCartID.! !

!Marketplace methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/12/2012 20:13'!
isAListedBook: anISBN  
	
	catalogue do: [:each | each = anISBN ifTrue: [^true]].
	
	^false.! !

!Marketplace methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/11/2012 21:58'!
setBooks: aCollectionOfBooks  
	
	catalogue  := aCollectionOfBooks.! !

!Marketplace methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/11/2012 21:49'!
setCarts: aCollectionOfCarts

	carts := aCollectionOfCarts.! !

TestCase subclass: #TusLibrosTest
	instanceVariableNames: 'marketplace'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/13/2012 18:55'!
cartWith: aCollectionOfBooks
	marketplace := Marketplace withBooks: aCollectionOfBooks.
	^ marketplace createCart! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/13/2012 18:57'!
test01CartsAreCreatedEmpty
	self assert: (self cartWith: #()) listContents isEmpty! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/13/2012 18:57'!
test02CantAddBookNotInCatalogToCart
	self should: [ (self cartWith: #()) addBook: 'test' withQuantity: 1 ] raise: BookNotInCatalogException! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/13/2012 18:57'!
test03CantAddZeroOrNegativeNumberOfBooks
	self should: [ (self cartWith: #()) addBook: 'test' withQuantity: 0 ] raise: NotAValidQuantityException.
	self should: [ (self cartWith: #()) addBook: 'test' withQuantity: -1 ] raise: NotAValidQuantityException! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/13/2012 18:56'!
test04ACanAddBookInCatalogue
	| cart list |
	cart := self cartWith: #('test' 'test2').
	cart addBook: 'test' withQuantity: 1.
	list := cart listContents.
	self assert: (list occurrencesOf: 'test') = 1! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/13/2012 18:56'!
test04BCanAddMultipleBooksMultipleTimes
	| cart list |
	cart := self cartWith: #('test' 'test2').
	cart addBook: 'test' withQuantity: 1.
	cart addBook: 'test' withQuantity: 3.
	cart addBook: 'test2' withQuantity: 2.
	list := cart listContents.
	self assert: (list occurrencesOf: 'test') = 4.
	self assert: (list occurrencesOf: 'test2') = 2! !
